@@ -9,10 +9,19 @@ from urllib.parse import urlparse
 import pytest
 import redis
 from packaging.version import Version
+from redis import Sentinel
 from redis.backoff import NoBackoff
-from redis.connection import Connection, parse_url
+from redis.cache import (
+    CacheConfig,
+    CacheFactoryInterface,
+    CacheInterface,
+    CacheKey,
+    EvictionPolicy,
+)
+from redis.connection import Connection, ConnectionInterface, SSLConnection, parse_url
 from redis.exceptions import RedisClusterException
 from redis.retry import Retry
+from tests.ssl_utils import get_ssl_filename
 
 REDIS_INFO = {}
 default_redis_url = "redis://localhost:6379/0"
@@ -71,6 +80,21 @@ class BooleanOptionalAction(argparse.Action):
         return " | ".join(self.option_strings)
 
 
+@pytest.fixture(scope="session", autouse=True)
+def enable_tracemalloc():
+    """
+    Enable tracemalloc while tests are being executed.
+    """
+    try:
+        import tracemalloc
+
+        tracemalloc.start()
+        yield
+        tracemalloc.stop()
+    except ImportError:
+        yield
+
+
 def pytest_addoption(parser):
     parser.addoption(
         "--redis-url",
@@ -105,6 +129,19 @@ def pytest_addoption(parser):
         "--uvloop", action=BooleanOptionalAction, help="Run tests with uvloop"
     )
 
+    parser.addoption(
+        "--sentinels",
+        action="store",
+        default="localhost:26379,localhost:26380,localhost:26381",
+        help="Comma-separated list of sentinel IPs and ports",
+    )
+    parser.addoption(
+        "--master-service",
+        action="store",
+        default="redis-py-test",
+        help="Name of the Redis master service that the sentinels are monitoring",
+    )
+
 
 def _get_info(redis_url):
     client = redis.Redis.from_url(redis_url)
@@ -232,7 +269,9 @@ def skip_ifmodversion_lt(min_version: str, module_name: str):
     for j in modules:
         if module_name == j.get("name"):
             version = j.get("ver")
-            mv = int(min_version.replace(".", ""))
+            mv = int(
+                "".join(["%02d" % int(segment) for segment in min_version.split(".")])
+            )
             check = version < mv
             return pytest.mark.skipif(check, reason="Redis module version")
 
@@ -290,8 +329,22 @@ def _get_client(
         kwargs["protocol"] = request.config.getoption("--protocol")
 
     cluster_mode = REDIS_INFO["cluster_enabled"]
+    ssl = kwargs.pop("ssl", False)
     if not cluster_mode:
         url_options = parse_url(redis_url)
+        connection_class = Connection
+        if ssl:
+            connection_class = SSLConnection
+            kwargs["ssl_certfile"] = get_ssl_filename("client-cert.pem")
+            kwargs["ssl_keyfile"] = get_ssl_filename("client-key.pem")
+            # When you try to assign "required" as single string
+            # it assigns tuple instead of string.
+            # Probably some reserved keyword
+            # I can't explain how does it work -_-
+            kwargs["ssl_cert_reqs"] = "require" + "d"
+            kwargs["ssl_ca_certs"] = get_ssl_filename("ca-cert.pem")
+            kwargs["port"] = 6666
+        kwargs["connection_class"] = connection_class
         url_options.update(kwargs)
         pool = redis.ConnectionPool(**url_options)
         client = cls(connection_pool=pool)
@@ -378,6 +431,41 @@ def sslclient(request):
         yield client
 
 
+@pytest.fixture()
+def sentinel_setup(request):
+    sentinel_ips = request.config.getoption("--sentinels")
+    sentinel_endpoints = [
+        (ip.strip(), int(port.strip()))
+        for ip, port in (endpoint.split(":") for endpoint in sentinel_ips.split(","))
+    ]
+    kwargs = request.param.get("kwargs", {}) if hasattr(request, "param") else {}
+    cache = request.param.get("cache", None)
+    cache_config = request.param.get("cache_config", None)
+    force_master_ip = request.param.get("force_master_ip", None)
+    decode_responses = request.param.get("decode_responses", False)
+    sentinel = Sentinel(
+        sentinel_endpoints,
+        force_master_ip=force_master_ip,
+        socket_timeout=0.1,
+        cache=cache,
+        cache_config=cache_config,
+        protocol=3,
+        decode_responses=decode_responses,
+        **kwargs,
+    )
+    yield sentinel
+    for s in sentinel.sentinels:
+        s.close()
+
+
+@pytest.fixture()
+def master(request, sentinel_setup):
+    master_service = request.config.getoption("--master-service")
+    master = sentinel_setup.master_for(master_service)
+    yield master
+    master.close()
+
+
 def _gen_cluster_mock_resp(r, response):
     connection = Mock(spec=Connection)
     connection.retry = Retry(NoBackoff(), 0)
@@ -454,6 +542,37 @@ def master_host(request):
     return parts.hostname, (parts.port or 6379)
 
 
+@pytest.fixture()
+def cache_conf() -> CacheConfig:
+    return CacheConfig(max_size=100, eviction_policy=EvictionPolicy.LRU)
+
+
+@pytest.fixture()
+def mock_cache_factory() -> CacheFactoryInterface:
+    mock_factory = Mock(spec=CacheFactoryInterface)
+    return mock_factory
+
+
+@pytest.fixture()
+def mock_cache() -> CacheInterface:
+    mock_cache = Mock(spec=CacheInterface)
+    return mock_cache
+
+
+@pytest.fixture()
+def mock_connection() -> ConnectionInterface:
+    mock_connection = Mock(spec=ConnectionInterface)
+    return mock_connection
+
+
+@pytest.fixture()
+def cache_key(request) -> CacheKey:
+    command = request.param.get("command")
+    keys = request.param.get("redis_keys")
+
+    return CacheKey(command, keys)
+
+
 def wait_for_command(client, monitor, command, key=None):
     # issue a command with a key name that's local to this process.
     # if we find a command with our key before the command we're waiting
